/* 
COUNTERSTORE

CounterStoressa säilytetään koko sovelluksen laskurin tilaa, 
eli numeroarvoa, jota kaikki komponentit voivat helposti käyttää 
ja päivittää. Esimerkissä on myös esitetty, kuinka alkutila
voitaisiin hakea tietokannasta.
*/

/* @ngrx/signals-kirjastosta saadut funktiot, joiden avulla
rakennetaan store */
import {
  patchState,
  signalStore,
  withHooks,
  withMethods,
  withState,
} from '@ngrx/signals';

import { inject } from '@angular/core';
import { DataService } from './data.service';

// laskurin alkutila, jota tarvitaan jos alkutila ei tule tietokannasta
const initialState = { count: 0 };

/* CounterStore määriteltynä funktionaalisesti.
   Se syntyy signalStore-funktiosta. 
   withState-funktion argumenttina on alkutila,
   withMethods-funktion argumenttina storen metodit
   withHooks-funktion argumenttina hookit, jotka 
   voivat laukaista esim. servicen metodin suorituksen
   tai efektejä, eli koko sovelluksessa tapahtuvia 
   reaktiivisia toimintoja.

   Store voi olla myös luokka, mutta funktionaalinen
   store on suositeltavampi. Luokan syntaksi on hieman
   erilainen kuin tässä esitetty syntaksi.
*/

export const CounterStore = signalStore(
  // Injektoitavissa kaikkiin komponentteihin
  { providedIn: 'root' },
  // alkutila alustetaan tässä, jos se ei tule tietokannasta
  withState(initialState),
  // storen metodit
  withMethods((store) => ({
    increment(): void {
      // patchstate-metodilla päivitetään storen tilaa
      patchState(store, (state) => ({ count: state.count + 1 }));
    },

    decrement(): void {
      patchState(store, (state) => ({ count: state.count - 1 }));
    },

    reset(): void {
      patchState(store, () => ({ count: 0 }));
    },
  })),
  // hookit eli lifecycle-metodit
  withHooks({
    // onInit suoritetaan aina kun store ladataan muistiin
    onInit(store, dataservice = inject(DataService)) {
      // datan hakeminen storeen tietokannasta tapahtuisi täällä 
    },
    // onDestroy suoritetaan aina kun store häviää muistista
    onDestroy(store) {
      console.log('ShopStore poistettu muistista', store);
    },
  })
);
